@@ -1,5 +1,4 @@
 /* ACME
- * init_tray (acme);
  * Copyright (C) 2001 Bastien Nocera <hadess@hadess.net>
  *
  * acme.c
@@ -30,10 +29,11 @@
 #include <X11/XF86keysym.h>
 /* Gnome headers */
 #include <gdk/gdkx.h>
-#include <gtk/gtkinvisible.h>
 #include <gnome.h>
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
+#include <libwnck/libwnck.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "eggtrayicon.h"
 
 #include "acme.h"
@@ -42,9 +42,6 @@
 #include "fb-level.h"
 #endif
 
-static gboolean event_cb (GtkWidget * widget, GdkEvent * event,
-			  gpointer data);
-
 typedef struct {
 	GVolume *volobj;
 #ifdef USE_FBLEVEL
@@ -52,10 +49,15 @@ typedef struct {
 #endif
 	GladeXML *xml;
 	GtkWidget *dialog;
-	GtkWidget *invisible;
 	int NumLockMask, CapsLockMask, ScrollLockMask;
 	GConfClient *conf_client;
 	guint dialog_timeout;
+	WnckScreen *screen;
+
+	/* Multihead stuff */
+	GdkDisplay *display;
+	GdkScreen *current_screen;
+	GList *screens;
 
 	/* Tray icon */
 	EggTrayIcon *tray_icon;
@@ -70,6 +72,8 @@ enum {
 	ICON_EJECT,
 };
 
+static void init_tray (Acme *acme);
+
 static void
 selection_get_func (GtkClipboard *clipboard, GtkSelectionData *selection_data,
 		guint info, gpointer user_data_or_owner)
@@ -178,7 +182,7 @@ execute_this_or_that (char *cmd1, char *cmd2)
 				(_("Couldn't execute either command: %s\n"
 				   "or command: %s\n"
 				   "Verify that at least one of these commands"
-				   "exist."),
+				   " exist."),
 				 cmd1, cmd2);
 
 			acme_error (msg);
@@ -241,6 +245,9 @@ fb_problem_cb (void)
 {
 	char *msg;
 
+	if (fb_level_is_powerbook () == FALSE)
+		return;
+
 	msg = permission_problem_string ("/dev/pmu");
 	acme_error (msg);
 	g_free (msg);
@@ -293,11 +300,12 @@ acme_image_set (Acme *acme, int icon)
 }
 
 static void
-ungrab_key (int key_code)
+ungrab_key_real (int key_code, GdkWindow *root)
 {
 	gdk_error_trap_push ();
 	XUngrabKey (GDK_DISPLAY (), key_code, AnyModifier,
-			GDK_ROOT_WINDOW ());
+			GDK_WINDOW_XID (root));
+
 	gdk_flush ();
 	if (gdk_error_trap_pop ()) {
 		char *error;
@@ -313,41 +321,56 @@ ungrab_key (int key_code)
 }
 
 static void
-grab_key (int key_code)
+ungrab_key (Acme *acme, int key_code)
+{
+	GList *l;
+
+	for (l = acme->screens; l != NULL; l = l->next)
+	{
+		GdkScreen *screen;
+
+		screen = (GdkScreen *) l->data;
+		ungrab_key_real (key_code, gdk_screen_get_root_window (screen));
+	}
+}
+
+
+static void
+grab_key_real (int key_code, GdkWindow *root)
 {
 	gdk_error_trap_push ();
 
 	XGrabKey (GDK_DISPLAY (), key_code,
 			0,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod2Mask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod5Mask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			LockMask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod2Mask | LockMask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod5Mask | LockMask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod2Mask | Mod5Mask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 	XGrabKey (GDK_DISPLAY (), key_code,
 			Mod2Mask | Mod5Mask | LockMask,
-			GDK_ROOT_WINDOW (), True,
+			GDK_WINDOW_XID (root), True,
 			GrabModeAsync, GrabModeAsync);
 
 	gdk_flush ();
@@ -363,15 +386,60 @@ grab_key (int key_code)
 			 key_code);
 		acme_error (error);
 		g_free (error);
-		exit (1);
 	}
 }
 
+static void
+grab_key (Acme *acme, int key_code)
+{
+	GList *l;
+
+	for (l = acme->screens; l != NULL; l = l->next)
+	{
+		GdkScreen *screen;
+
+		screen = (GdkScreen *) l->data;
+		grab_key_real (key_code, gdk_screen_get_root_window (screen));
+	}
+}
+
+static void
+unhookup_keysym (int keycode)
+{
+	char *command;
+
+	if (keycode <= 0)
+		return;
+
+	command = g_strdup_printf ("xmodmap -e \"keycode %d = \"", keycode);
+
+	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
+	g_free (command);
+}
+
+static gboolean
+hookup_keysym (int keycode, const char *keysym)
+{
+	char *command;
+
+	if (keycode <= 0)
+		return TRUE;
+
+	command = g_strdup_printf ("xmodmap -e \"keycode %d = %s\"",
+			keycode, keysym);
+
+	g_spawn_command_line_sync (command, NULL, NULL, NULL, NULL);
+	g_free (command);
+
+	return FALSE;
+}
+
 static void
 update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 {
 	Acme *acme = (Acme *) data;
 	int i;
+	char *keysym = NULL;
 
 	g_return_if_fail (entry->key != NULL);
 
@@ -381,14 +449,55 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 		if (strcmp (entry->key, keys[i].key_config) == 0)
 		{
 			int key_code;
+			gboolean grab = TRUE;
+
+			ungrab_key (acme, keys[i].key_code);
+			switch (keys[i].key_type) {
+			case PLAY_KEY:
+			case PAUSE_KEY:
+			case STOP_KEY:
+			case PREVIOUS_KEY:
+			case NEXT_KEY:
+			case REFRESH_KEY:
+				unhookup_keysym (keys[i].key_code);
+				break;
+			}
+
+			keys[i].key_code = -1;
 
-			ungrab_key (keys[i].key_code);
 			key_code = gconf_client_get_int (acme->conf_client,
-					keys[i].key_config,
-					NULL);
-			if (key_code > 0)
+					keys[i].key_config, NULL);
+
+			switch (keys[i].key_type) {
+			case PLAY_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86AudioPlay");
+				break;
+			case PAUSE_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86AudioPause");
+				break;
+			case STOP_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86AudioStop");
+				break;
+			case PREVIOUS_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86AudioPrev");
+				break;
+			case NEXT_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86AudioNext");
+				break;
+			case REFRESH_KEY:
+				grab = hookup_keysym (key_code,
+						"XF86Refresh");
+				break;
+			}
+
+			if (key_code > 0 && grab == TRUE)
 			{
-				grab_key (key_code);
+				grab_key (acme, key_code);
 				keys[i].key_code = key_code;
 			}
 		}
@@ -398,20 +507,37 @@ update_kbd_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
 static gboolean
 init_kbd (Acme *acme)
 {
-	GtkWidget *widget = acme->invisible;
 	int i;
 	gboolean retval = FALSE;
 
-	gtk_widget_show (widget);
+	acme->display = gdk_display_get_default ();
+	acme->screens = NULL;
+#ifdef ENABLE_MULTIHEAD
+	if (gdk_display_get_n_screens (acme->display) == 1)
+	{
+		acme->screens = g_list_append (acme->screens,
+				gdk_screen_get_default ());
+	} else {
+		for (i = 0; i < gdk_display_get_n_screens (acme->display); i++)
+		{
+			GdkScreen *screen;
 
-	/* Make the root window send events to the invisible proxy widget */
-	gdk_window_set_user_data (gdk_get_default_root_window (), widget);
+			screen = gdk_display_get_screen (acme->display, i);
 
-	gdk_window_set_events (gdk_get_default_root_window (),
-			GDK_KEY_PRESS_MASK);
+			if (screen != NULL)
+				acme->screens = g_list_append (acme->screens,
+						screen);
+		}
+	}
+
+#else
+	acme->screens = g_list_append (acme->screens,
+			gdk_screen_get_default ());
+#endif
 
 	for (i = 0; i < HANDLED_KEYS; i++)
 	{
+		gboolean grab = TRUE;
 		int tmp;
 
 		tmp = gconf_client_get_int (acme->conf_client,
@@ -420,14 +546,41 @@ init_kbd (Acme *acme)
 
 		keys[i].key_code = tmp;
 
-		if (tmp > 0)
+		switch (keys[i].key_type) {
+		case PLAY_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86AudioPlay");
+			break;
+		case PAUSE_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86AudioPause");
+			break;
+		case STOP_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86AudioStop");
+			break;
+		case PREVIOUS_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86AudioPrev");
+			break;
+		case NEXT_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86AudioNext");
+			break;
+		case REFRESH_KEY:
+			grab = hookup_keysym (keys[i].key_code,
+					"XF86Refresh");
+			break;
+		}
+
+		if (tmp > 0 && grab == TRUE)
 		{
 #ifdef DEBUG
 			g_print ("grabbed key %d for gconf key %s\n",
 					keys [i].key_code,
 					keys[i].key_config);
 #endif
-			grab_key (keys [i].key_code);
+			grab_key (acme, keys [i].key_code);
 			retval = TRUE;
 		}
 	}
@@ -465,20 +618,22 @@ about_activated (GtkMenuItem *menuitem, gpointer user_data)
 		return;
 	}
 
-	pixbuf = gdk_pixbuf_new_from_file (ACME_DATA "keymap-48.png", NULL);
+	pixbuf = gdk_pixbuf_new_from_file (ACME_DATA "acme-48.png", NULL);
 
 	about = gnome_about_new(_("Acme"), VERSION,
-			_("(C) 2001, 2002 Bastien Nocera"),
+			"Copyright \xc2\xa9 2001-2002 Bastien Nocera",
 			_("Multimedia keys daemon"),
 			(const char **)authors,
 			(const char **)documenters,
 			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 			pixbuf);
-
-	g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK
-			(gtk_widget_destroyed), &about);
-	g_object_add_weak_pointer (G_OBJECT (about),
-			(void**)&(about));
+	
+	if (pixbuf != NULL)
+		gdk_pixbuf_unref (pixbuf);
+	
+	g_signal_connect (G_OBJECT (about), "destroy",
+			G_CALLBACK (gtk_widget_destroyed), &about);
+	g_object_add_weak_pointer (G_OBJECT (about), (void**)&(about));
 
 	gtk_widget_show(about);
 }
@@ -509,13 +664,20 @@ tray_icon_press (GtkWidget *widget, GdkEventButton *event, Acme *acme)
 	return FALSE;
 }
 
+static gboolean
+tray_destroyed (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+{
+	init_tray ((Acme *) user_data);
+	return TRUE;
+}
+
 static void
 init_tray (Acme *acme)
 {
 	GtkWidget *image, *evbox, *item;
 
 	acme->tray_icon = egg_tray_icon_new ("Multimedia Keys daemon");
-	image = gtk_image_new_from_file (ACME_DATA "keymap-16.png");
+	image = gtk_image_new_from_file (ACME_DATA "acme-16.png");
 
 	acme->tray_icon_tooltip = gtk_tooltips_new ();
 	gtk_tooltips_set_tip (acme->tray_icon_tooltip,
@@ -549,6 +711,9 @@ init_tray (Acme *acme)
 	gtk_container_add (GTK_CONTAINER (evbox), image);
 	gtk_container_add (GTK_CONTAINER (acme->tray_icon), evbox);
 	gtk_widget_show_all (GTK_WIDGET (acme->tray_icon));
+
+	g_signal_connect (G_OBJECT (acme->tray_icon), "destroy-event",
+			G_CALLBACK (tray_destroyed), (gpointer) acme);
 }
 
 static void
@@ -571,7 +736,7 @@ init_sm (Acme *acme)
 	}
 
 	g_signal_connect (GTK_OBJECT (master), "die",
-			(GtkSignalFunc) acme_exit, acme);
+			G_CALLBACK (acme_exit), acme);
 }
 
 static gboolean
@@ -589,11 +754,22 @@ dialog_show (Acme *acme)
 	int screen_w, screen_h;
 	int x, y;
 
+#ifdef ENABLE_MULTIHEAD
+	gtk_window_set_screen (GTK_WINDOW (acme->dialog), acme->current_screen);
+#endif
+	gtk_widget_realize (GTK_WIDGET (acme->dialog));
+
 	gdk_window_get_geometry (GTK_WIDGET (acme->dialog)->window,
 				 &orig_x, &orig_y,
 				 &orig_w, &orig_h, &orig_d);
+
+#ifdef ENABLE_MULTIHEAD
+	screen_w = gdk_screen_get_width (acme->current_screen);
+	screen_h = gdk_screen_get_height (acme->current_screen);
+#else
 	screen_w = gdk_screen_width ();
 	screen_h = gdk_screen_height ();
+#endif
 
 	x = (screen_w - orig_w) / 2;
 	y = screen_h / 2 + (screen_h / 2 - orig_h) / 2;
@@ -610,6 +786,86 @@ dialog_show (Acme *acme)
 			(GtkFunction) dialog_hide, acme);
 }
 
+static void
+do_close_window_action (Acme *acme)
+{
+	GList *windows, *item;
+	WnckWindow *focused;
+
+	focused = NULL;
+	windows = wnck_screen_get_windows (acme->screen);
+
+	if (windows == NULL)
+		return;
+
+	for (item = windows; item != NULL; item = item->next)
+	{
+		WnckWindow *window = (WnckWindow *) (item->data);
+		if (wnck_window_is_active (window) == TRUE)
+		{
+			focused = window;
+			break;
+		}
+	}
+
+	if (focused != NULL)
+		wnck_window_close (focused);
+}
+
+static void
+do_shade_window_action (Acme *acme)
+{
+	GList *windows, *item;
+	WnckWindow *focused;
+
+	focused = NULL;
+	windows = wnck_screen_get_windows (acme->screen);
+
+	if (windows == NULL)
+		return;
+
+	for (item = windows; item != NULL; item = item->next)
+	{
+		WnckWindow *window = (WnckWindow *) (item->data);
+		if (wnck_window_is_active (window) == TRUE)
+		{
+			focused = window;
+			break;
+		}
+	}
+
+	if (focused != NULL)
+	{
+		if (wnck_window_is_shaded (focused) == TRUE)
+			wnck_window_unshade (focused);
+		else
+			wnck_window_shade (focused);
+	}
+}
+
+static void
+do_www_action (Acme *acme, const char *url)
+{
+	char *string, *command;
+
+	string = gconf_client_get_string (acme->conf_client,
+		"/desktop/gnome/url-handlers/unknown/command",
+		 NULL);
+
+	if (string == NULL && strcmp (string, "") == 0)
+		return;
+
+	if (url == NULL)
+		command = g_strdup_printf (string, "about:blank");
+	else
+		command = g_strdup_printf (string, url);
+
+	execute (command, FALSE);
+
+	g_free (command);
+	g_free (string);
+}
+
 static void
 do_exit_action (Acme *acme)
 {
@@ -656,6 +912,18 @@ do_eject_action (Acme *acme)
 	gtk_widget_set_sensitive (progress, TRUE);
 }
 
+static void
+do_media_action (Acme *acme)
+{
+	GnomeVFSMimeApplication *app;
+
+	app = gnome_vfs_mime_get_default_application ("audio/x-mp3");
+	if (app->requires_terminal == TRUE || app->command == NULL)
+		return;
+
+	execute (app->command, FALSE);
+}
+
 #ifdef USE_FBLEVEL
 static void
 do_brightness_action (Acme *acme, int type)
@@ -760,7 +1028,18 @@ do_action (int type, Acme *acme)
 		do_exit_action (acme);
 		break;
 	case EJECT_KEY:
-		/* Do nothing, handled differently */
+		do_eject_action (acme);
+		break;
+	case MEDIA_KEY:
+		do_media_action (acme);
+		break;
+	/* For GCC to be happy */
+	case PLAY_KEY:
+	case PAUSE_KEY:
+	case STOP_KEY:
+	case PREVIOUS_KEY:
+	case NEXT_KEY:
+	case REFRESH_KEY:
 		break;
 	case HOME_KEY:
 		execute ("nautilus", FALSE);
@@ -769,7 +1048,7 @@ do_action (int type, Acme *acme)
 		execute ("gnome-search-tool", FALSE);
 		break;
 	case EMAIL_KEY:
-		execute_this_or_that ("evolution", "balsa");
+		execute_this_or_that ("evolution mailto:", "balsa");
 		break;
 	case SLEEP_KEY:
 		execute ("apm", TRUE);
@@ -783,6 +1062,24 @@ do_action (int type, Acme *acme)
 	case HELP_KEY:
 		execute ("yelp", FALSE);
 		break;
+	case WWW_KEY:
+		do_www_action (acme, NULL);
+		break;
+	case GROUPS_KEY:
+		do_www_action (acme, "http://www.gnomedesktop.org");
+		break;
+	case CALCULATOR_KEY:
+		execute ("gnome-calculator", FALSE);
+		break;
+	case RECORD_KEY:
+		execute ("gnome-sound-recorder", FALSE);
+		break;
+	case CLOSE_WINDOW_KEY:
+		do_close_window_action (acme);
+		break;
+	case SHADE_WINDOW_KEY:
+		do_shade_window_action (acme);
+		break;
 #ifdef USE_FBLEVEL
 	case BRIGHT_DOWN_KEY:
 	case BRIGHT_UP_KEY:
@@ -795,65 +1092,73 @@ do_action (int type, Acme *acme)
 	}
 }
 
-static void
-check_eject (Acme *acme, GdkEvent *event)
-{
-	GdkEventKey *key;
-	static guint eject_press_time = 0;
-
-	key = (GdkEventKey *) event;
-
-#ifdef DEBUG
-	g_print ("type %d (press %d release %d)\n", event->type, GDK_KEY_PRESS, GDK_KEY_RELEASE);
-	if (event->type == GDK_KEY_PRESS || event->type == GDK_KEY_RELEASE)
-		g_print ("key pressed: %d\n", key->hardware_keycode);
-#endif
-
-	if (event->type == GDK_KEY_PRESS
-			&& key->hardware_keycode == keys[EJECT_KEY].key_code
-			&& eject_press_time == 0)
-		eject_press_time = key->time;
-	if (event->type == GDK_KEY_RELEASE
-			&& key->hardware_keycode == keys[EJECT_KEY].key_code)
-	{
-		if (key->time - eject_press_time >= EJECT_TIMEOUT)
-			do_eject_action (acme);
-		eject_press_time = 0;
-	}
-}
-
-static gboolean
-event_cb (GtkWidget * widget, GdkEvent * event, gpointer data)
+static GdkFilterReturn
+acme_filter_events (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 {
 	Acme *acme = (Acme *) data;
-	GdkEventKey *key;
+	XEvent *xev = (XEvent *) xevent;
+	XAnyEvent *xanyev = (XAnyEvent *) xevent;
+	XKeyEvent *key = (XKeyEvent *) xevent;
+	GdkScreen *event_screen = NULL;
+	GList *l;
 	int i;
 
-	/* Eject is handled differently */
-	check_eject (acme, event);
+	if (xev->type != KeyRelease)
+		return GDK_FILTER_CONTINUE;
 
-	if (event->type != GDK_KEY_PRESS)
-		return TRUE;
+	/* Look for which screen we're receiving events */
+	for (l = acme->screens; (l != NULL) && (event_screen == NULL);
+			l = l->next)
+	{
+		GdkWindow *window;
+		GdkScreen *screen;
 
-	key = (GdkEventKey *) event;
+		screen = (GdkScreen *) l->data;
+		window = gdk_screen_get_root_window (screen);
 
-	for (i = 0; i < HANDLED_KEYS; i++) {
-#ifdef DEBUG
-		g_print ("comparing %d and %d\n", keys[i].key_code,
-				key->hardware_keycode);
-#endif
-		if (keys[i].key_code == key->hardware_keycode) {
-			do_action (keys[i].key_type, acme);
+		if (GDK_WINDOW_XID (window) == xanyev->window)
+		{
+			event_screen = screen;
 			break;
 		}
 	}
-	return TRUE;
+
+	key = (XKeyEvent *) xevent;
+
+	for (i = 0; i < HANDLED_KEYS; i++)
+	{
+#ifdef DEBUG
+		g_print ("comparing %d and %d\n", keys[i].key_code,
+				key->keycode);
+#endif
+
+		if (keys[i].key_code == key->keycode)
+		{
+			switch (keys[i].key_type) {
+			case PLAY_KEY:
+			case PAUSE_KEY:
+			case STOP_KEY:
+			case PREVIOUS_KEY:
+			case NEXT_KEY:
+			case REFRESH_KEY:
+				return GDK_FILTER_CONTINUE;
+			}
+
+			acme->current_screen = event_screen;
+
+			do_action (keys[i].key_type, acme);
+			return GDK_FILTER_REMOVE;
+		}
+	}
+
+	return GDK_FILTER_CONTINUE;
 }
 
 int
 main (int argc, char *argv[])
 {
 	Acme *acme;
+	GList *l;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -892,7 +1197,6 @@ main (int argc, char *argv[])
 			GCONF_CLIENT_PRELOAD_ONELEVEL,
 			NULL);
 
-	acme->invisible = gtk_invisible_new ();
 	if (init_kbd (acme) == FALSE)
 	{
 		execute ("acme-properties", FALSE);
@@ -900,6 +1204,8 @@ main (int argc, char *argv[])
 	}
 	init_sm (acme);
 	init_tray (acme);
+	acme->current_screen = gdk_screen_get_default ();
+	acme->screen = wnck_screen_get_default ();
 	gtk_widget_realize (acme->dialog);
 	acme->dialog_timeout = 0;
 
@@ -927,9 +1233,18 @@ main (int argc, char *argv[])
 		fb_problem_cb ();
 #endif
 
-	g_signal_connect (GTK_OBJECT (acme->invisible),
-			    "event",
-			    G_CALLBACK (event_cb), acme);
+	/* Start filtering the events */
+	for (l = acme->screens; l != NULL; l = l->next)
+	{
+		GdkScreen *screen;
+		GdkWindow *window;
+
+		screen = (GdkScreen *) l->data;
+		window = gdk_screen_get_root_window (screen);
+		gdk_window_add_filter (window,
+				acme_filter_events,
+				(gpointer) acme);
+	}
 
 	gtk_main ();
 
